/// CreateGUI.cpp - The method from NPBWindow used to create the GUI from the
///					xml file.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2004-2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------
///	
///	You can e-mail me at niallmoody@yahoo.co.uk if you want.
///	
///	--------------------------------------------------------------------------

#include "NPBWindow.h"
#include "ComponentFactory.h"
#include "WindowComponents/MainWindow.h"
#include "WindowComponents/CloseButton.h"
#include "WindowComponents/MinimiseButton.h"
#include "WindowComponents/MaximiseButton.h"
#include "WindowComponents/FileMenu.h"
#include "WindowComponents/OptionsMenu.h"
#include "WindowComponents/HelpMenu.h"
#include "WindowComponents/PluginOrganiser.h"
#include "WindowComponents/AudioButton.h"
#include "WindowComponents/OptionsButton.h"
#include "WindowComponents/RecordButton.h"

///---------------------------------------------------------------------------
void NPBWindow::createGUI()
{
	//Shouldn't happen, but just in case...
	if(!factory)
		return;

	//--Create any FactoryObjects here.--
	//CloseButton *cb = new CloseButton(factory, reinterpret_cast<Component **>(&closeButton));
	MaximiseButton *mb = new MaximiseButton(factory, reinterpret_cast<Component **>(&maxButton));
	MinimiseButton *mnb = new MinimiseButton(factory, reinterpret_cast<Component **>(&minButton));
	FileMenuComp *fm = new FileMenuComp(factory, reinterpret_cast<Component **>(&FileMenu));
	OptionsMenuComp *om = new OptionsMenuComp(factory, reinterpret_cast<Component **>(&OptionsMenu));
	HelpMenuComp *hm = new HelpMenuComp(factory, reinterpret_cast<Component **>(&HelpMenu));
	AudioButtonComp *ab = new AudioButtonComp(factory, reinterpret_cast<Component **>(&AudioButton));
	OptionsButtonComp *ob = new OptionsButtonComp(factory, reinterpret_cast<Component **>(&OptionsButton));
	RecordButtonComp *rb = new RecordButtonComp(factory, reinterpret_cast<Component **>(&RecordButton));
	PluginOrganiser *po = new PluginOrganiser(factory, reinterpret_cast<Component **>(&Organiser));

	//Always leave this last, as it resizes the whole window.
	MainWindow *window = new MainWindow(factory, 0);

	//Is it okay for this to be here?
	//(it's here so that the LookAndFeel gets set right)
	CloseButton *cb = new CloseButton(factory, reinterpret_cast<Component **>(&closeButton));

	factory->createComponents();
}
