//	NPBMIDIDialog.cpp - Dialog for various VST-specific options.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBMIDIDialog.h"
#include "NiallsTB.h"

NPBMIDIDialog::NPBMIDIDialog(NPBWindow *parent, Colour col):
DialogWindow(T("MIDI Options"), col)
{
	setVisible(true);

	NPBMIDIContent *cont = new NPBMIDIContent(parent, col);
	setContentComponent(cont);
}

//--------------------------------------------------------------------------
NPBMIDIContent::NPBMIDIContent(NPBWindow *parent, Colour col):
Component()
{
	int i;
	String tempstr;
	StringArray temparr;

	this->parent = parent;

	addAndMakeVisible(deviceLabel = new Label(T("deviceLabel"), T("Device: ")));
	addAndMakeVisible(device = new ComboBox(T("device")));
	temparr = MidiInput::getDevices();
	for(i=0;i<temparr.size();i++)
		device->addItem(temparr[i], i);
	device->setSelectedId(0);

	Colour tempc = col.darker(0.21f);
	Colour temp2 = col.darker(0.61f);
	addAndMakeVisible(ok = new NiallsTB(T("OK"), T("Yay!")));
	ok->setBackgroundColours(tempc, Colours::white);
	ok->addActionListener(this);

	addAndMakeVisible(cancel = new NiallsTB(T("Cancel"), T("Boo!")));
	cancel->setBackgroundColours(tempc, Colours::white);
	cancel->addActionListener(this);
}

void NPBMIDIContent::actionListenerCallback(const String& m)
{
	if(m == ok->getName())
	{
		parent->setMIDIInputDevice(device->getSelectedId());
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
}

void NPBMIDIContent::resized()
{
	deviceLabel->setBounds(10, 10, 60, 25);
	device->setBounds(65, 10, 175, 25);

	ok->setBounds(10, (getHeight()-31), 100, 25);
	cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
}
