//	NPBMIDIDialog.h - Dialog for various VST-specific options.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBMIDIDIALOG_H_
#define NPBMIDIDIALOG_H_

#ifndef NPBWINDOW_H_
#include "NPBWindow.h"
#endif

#include <juce.h>

///	Dialog allowing the user to select a MIDI interface for input.
class NPBMIDIDialog : public DialogWindow
{
  public:
	///	Constructor.
	/*!
		\param parent The parent NPBWindow, used to let the main program know
		the MIDI settings have changed.
		\param col The background colour for the window - used by the
		skinning/scripting methods.
	 */
	NPBMIDIDialog(NPBWindow *parent, Colour col);
	///	Destructor.
	~NPBMIDIDialog() {};
};

///	The contents of the NPBMIDIDialog DialogWindow.
class NPBMIDIContent : public Component,
						   public ActionListener
{
  public:
	///	Constructor.
	/*!
		\param parent The parent NPBWindow, used to let the main program know
		the MIDI settings have changed.
		\param col The background colour for the window - used by the
		skinning/scripting methods.
	 */
	NPBMIDIContent(NPBWindow *parent, Colour col);
	///	Destructor.
	~NPBMIDIContent() {};

	///	Used to determine when the user has pushed the OK and Cancel buttons.
	void actionListenerCallback(const String& m);
	///	Used to place the various Components correctly.
	void resized();
  private:
	NPBWindow *parent;

	Label *deviceLabel;
	ComboBox *device;

	TextButton *ok;
	TextButton *cancel;
};

#endif
