//	NPBMappingsDialog.cpp - Dialog window which holds all the mappings
//						    currently set up.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBMappingsDialog.h"
#include "NiallsTB.h"

NPBMappingsDialog::NPBMappingsDialog(NPBPlugin *parent, OwnedArray<NPBMapping> *array, Colour col):
DialogWindow(T("Mappings"), col)
{
	setVisible(true);

	setContentComponent(new NPBMappingsContent(parent, array, col));
}

//------------------------------------------------------------------
NPBMappingsContent::NPBMappingsContent(NPBPlugin *parent, OwnedArray<NPBMapping> *array, Colour col):
Component(),
backColour(col)
{
	int i;
	NPBMapping *map;

	Parent = parent;
	for(i=0;i<array->size();i++)
	{
		map = new NPBMapping;
		map->Index = (*array)[i]->Index;
		map->JoyControl = (*array)[i]->JoyControl;
		map->JoyIndex = (*array)[i]->JoyIndex;
		map->MinVal = (*array)[i]->MinVal;
		map->MaxVal = (*array)[i]->MaxVal;
		map->PluginParameter = (*array)[i]->PluginParameter;
		map->parameterType = (*array)[i]->parameterType;
		map->MIDI = (*array)[i]->MIDI;
		map->MIDINote = (*array)[i]->MIDINote;
		MappingsArray.add(map);
	}

	Colour tempc = backColour.darker(0.21f);
	Colour temp2 = backColour.darker(0.61f);
	addAndMakeVisible(AddParameter = new NiallsTB(T("Add Parameter"), T("Add a mapping between joystick and this plugin's parameter.")));
	AddParameter->setBackgroundColours(tempc, Colours::white);
	AddParameter->addActionListener(this);

	addAndMakeVisible(AddMIDI = new NiallsTB(T("Add MIDI"), T("Add a mapping between joystick and this plugin's parameter.")));
	AddMIDI->setBackgroundColours(tempc, Colours::white);
	AddMIDI->addActionListener(this);

	addAndMakeVisible(Remove = new NiallsTB(T("Remove"), T("Remove the selected mapping.")));
	Remove->setBackgroundColours(tempc, Colours::white);
	Remove->addActionListener(this);

	addAndMakeVisible(Mappings = new SimpleListBox(T("Mappings"), this));
	Mappings->setOutlineColour(Colours::cadetblue, 1);
    Mappings->setRowHeight(24);

	addAndMakeVisible(OK = new NiallsTB(T("OK"), T("Yay!")));
	OK->setBackgroundColours(tempc, Colours::white);
	OK->addActionListener(this);

	addAndMakeVisible(Cancel = new NiallsTB(T("Cancel"), T("Boo!")));
	Cancel->setBackgroundColours(tempc, Colours::white);
	Cancel->addActionListener(this);

	addAndMakeVisible(acceptMIDI = new ToggleButton(T("Accept MIDI Input")));
	acceptMIDI->setToggleState(parent->getAcceptMIDI(), false);
}

NPBMappingsContent::~NPBMappingsContent()
{
	//Because the pointers in MappingsArray are also in the NPBPlugin's MappingsArray
	//MappingsArray.clear(false);
}

void NPBMappingsContent::paint(Graphics& g)
{
	SolidColourBrush brush(backColour);
	g.setBrush(&brush);
	g.fillAll();
}

void NPBMappingsContent::actionListenerCallback(const String& m)
{
	if(m == OK->getName())
	{
		Parent->setMappings(&MappingsArray);
		Parent->setAcceptMIDI(acceptMIDI->getToggleState());
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == Cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	else if(m == AddParameter->getName())
	{
		NPBMappingAdd *dlg = new NPBMappingAdd(this, Parent, backColour);
		dlg->addToDesktop(windowHasDropShadow);

		dlg->centreWithSize(450, 100);
		dlg->enterModalState();
	}
	else if(m == AddMIDI->getName())
	{
		NPBMappingAddMIDI *dlg = new NPBMappingAddMIDI(this, Parent, backColour);
		dlg->addToDesktop(windowHasDropShadow);

		dlg->centreWithSize(560, 100);
		dlg->enterModalState();
	}
	else if(m == Remove->getName())
	{
		if(Mappings->getSelectedRow() != -1)
		{
			MappingsArray.remove(Mappings->getSelectedRow());
			Mappings->updateContent();
		}
	}
}

void NPBMappingsContent::resized()
{
	AddParameter->setBounds(10, 5, 100, 25);
	AddMIDI->setBounds(120, 5, 100, 25);
	Remove->setBounds(230, 5, 100, 25);

	Mappings->setBounds(10, 40, (getWidth()-20), (getHeight()-81));

	OK->setBounds(10, (getHeight()-31), 100, 25);
	Cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
	acceptMIDI->setBounds(125, (getHeight()-31), 150, 25);
}

int NPBMappingsContent::getNumRows()
{
	return MappingsArray.size();
}

void NPBMappingsContent::paintListBoxItem(int rowNumber, Graphics &g, int width, int height, bool rowIsSelected)
{
	String tempstr;
	String tempstr2;

	if(rowIsSelected)
		g.fillAll (Colours::aquamarine);

	if(MappingsArray.size() > 0)
	{
		tempstr.printf("Joy %d : ", MappingsArray[rowNumber]->JoyIndex);
		switch(MappingsArray[rowNumber]->JoyControl)
		{
			case evAxis:
				tempstr += "Axis : ";
				break;
			case evRotation:
				tempstr += "Rotation : ";
				break;
			case evSlider:
				tempstr += "Slider : ";
				break;
			case evPOV:
				tempstr += "POV : ";
				break;
			case evButton:
				tempstr += "Button : ";
				break;
			case evVelocity:
				tempstr += "Velocity : ";
				break;
			case evAngularVelocity:
				tempstr += "Angular Velocity : ";
				break;
			case evSliderVelocity:
				tempstr += "Slider Velocity : ";
				break;
			case evAcceleration:
				tempstr += "Acceleration : ";
				break;
			case evAngularAcceleration:
				tempstr += "Angular Acceleration : ";
				break;
			case evSliderAcceleration:
				tempstr += "Slider Acceleration : ";
				break;
			case evForce:
				tempstr += "Force : ";
				break;
			case evTorque:
				tempstr += "Torque : ";
				break;
			case evSliderForce:
				tempstr += "Slider Force : ";
				break;
			case evEnvelopeDetector:
				tempstr += "Envelope Detector : ";
				break;
			case evMIDICC:
				tempstr += "MIDI CC : ";
				break;
			case evMIDINoteVal:
				tempstr += "MIDI Note Value : ";
				break;
			case evMIDIVelocity:
				tempstr += "MIDI Velocity : ";
				break;
		}
		tempstr << MappingsArray[rowNumber]->Index << "          ";
		tempstr << MappingsArray[rowNumber]->MinVal << "          ";
		tempstr << MappingsArray[rowNumber]->MaxVal << "          ";
		/*if(MappingsArray[rowNumber]->PluginParameter != -1)
		{*/
		if(MappingsArray[rowNumber]->parameterType)
		{
			if(MappingsArray[rowNumber]->PluginParameter < Parent->getNumParameters_vst())
				Parent->getParameterName(MappingsArray[rowNumber]->PluginParameter, tempstr2);
			else if(MappingsArray[rowNumber]->PluginParameter == Parent->getNumParameters_vst())
				tempstr2 = T("Bypass");
			else if(MappingsArray[rowNumber]->PluginParameter == (Parent->getNumParameters_vst()+1))
				tempstr2 = T("Mix");
		}
		else
		{
			if(MappingsArray[rowNumber]->MIDI == 128)
				tempstr2 << T("MIDI Velocity");
			else if(MappingsArray[rowNumber]->MIDI == 129)
				tempstr2 << T("MIDI Pitch Bend");
			else
			{
				tempstr2 << "MIDI CC ";
				tempstr2 << MappingsArray[rowNumber]->MIDI;
			}

			tempstr2 << "    Note ";
			tempstr2 << static_cast<int>(MappingsArray[rowNumber]->MIDINote);
		}
		/*}
		else
		{
			switch(MappingsArray[rowNumber]->Key)
			{
				case 0:
					tempstr2 = "key a";
					break;
				case 1:
					tempstr2 = "key b";
					break;
				case 2:
					tempstr2 = "key c";
					break;
				case 3:
					tempstr2 = "key d";
					break;
				case 4:
					tempstr2 = "key e";
					break;
				case 5:
					tempstr2 = "key f";
					break;
				case 6:
					tempstr2 = "key g";
					break;
				case 7:
					tempstr2 = "key h";
					break;
				case 8:
					tempstr2 = "key i";
					break;
				case 9:
					tempstr2 = "key j";
					break;
				case 10:
					tempstr2 = "key k";
					break;
				case 11:
					tempstr2 = "key l";
					break;
				case 12:
					tempstr2 = "key m";
					break;
				case 13:
					tempstr2 = "key n";
					break;
				case 14:
					tempstr2 = "key o";
					break;
				case 15:
					tempstr2 = "key p";
					break;
				case 16:
					tempstr2 = "key q";
					break;
				case 17:
					tempstr2 = "key r";
					break;
				case 18:
					tempstr2 = "key s";
					break;
				case 19:
					tempstr2 = "key t";
					break;
				case 20:
					tempstr2 = "key u";
					break;
				case 21:
					tempstr2 = "key v";
					break;
				case 22:
					tempstr2 = "key w";
					break;
				case 23:
					tempstr2 = "key x";
					break;
				case 24:
					tempstr2 = "key y";
					break;
				case 25:
					tempstr2 = "key z";
					break;
			}
		}*/
		tempstr += tempstr2;
		g.drawText(tempstr, 
				   4, 0, width - 4, height, 
				   Justification::centredLeft, true);
	}
	/*else //just for testing...
	{
		tempstr.printf("Joy 0 : ");		
		tempstr += "Angular Acceleration : ";
		tempstr << "0          ";
		tempstr << "0.0          ";
		tempstr << "1.0          ";
		Parent->getParameterName(0, tempstr2);
		tempstr += tempstr2;
		g.drawText(tempstr, 
				   4, 0, width - 4, height, 
				   Justification::centredLeft, true);
	}*/
}

void NPBMappingsContent::addMapping(NPBMapping *map)
{
	getParentComponent()->toFront(true);

	MappingsArray.add(map);
	Mappings->updateContent();
}

//---------------------------------------------------------
NPBMappingAdd::NPBMappingAdd(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col):
DialogWindow(T("Add a Parameter Mapping"), col)
{
	setVisible(true);

	setContentComponent(new NPBAddContent(parent, plugin, col));
}

//---------------------------------------------------------
NPBAddContent::NPBAddContent(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col):
Component(),
backColour(col)
{
	int i;
	String tempstr;

	Parent = parent;
	Plugin = plugin;

	addAndMakeVisible(JoyNum = new ComboBox(T("JoyNum")));
	for(i=0;i<Plugin->getNumJoysticks();i++)
	{
		tempstr.printf("%d", i);
		JoyNum->addItem(tempstr, (i+1));
	}
	JoyNum->setSelectedId(1, true);

	addAndMakeVisible(JoyControl = new NiallsLabel(T("JoyControl"), T("click me")));
	JoyControl->addActionListener(this);
	JoyControl->setColours(Colours::black, Colours::white, Colours::black, Colours::white, Colours::black, Colours::white);

	addAndMakeVisible(MinVal = new TextEditor(T("0.0")));
	MinVal->setInputRestrictions(7, T(".0123456789"));
	MinVal->setText(T("0.0"));

	addAndMakeVisible(MaxVal = new TextEditor(T("1.0")));
	MaxVal->setInputRestrictions(7, T(".0123456789"));
	MaxVal->setText(T("1.0"));

	addAndMakeVisible(PluginParameter = new ComboBox(T("PluginParameter")));
	for(i=0;i<Plugin->getNumParameters_vst();i++)
	{
		Plugin->getParameterName(i, tempstr);
		PluginParameter->addItem(tempstr, (i+1));
	}
	i++;
	PluginParameter->addItem(T("Bypass (npb)"), i);
	i++;
	PluginParameter->addItem(T("Mix (npb)"), i);
	/*mixId = i;
	setupComboBox(PluginParameter, i);*/
	PluginParameter->setSelectedId(1, true);

	Colour tempc = backColour.darker(0.21f);
	Colour temp2 = backColour.darker(0.61f);
	addAndMakeVisible(OK = new NiallsTB(T("OK"), T("Yay!")));
	OK->setBackgroundColours(tempc, Colours::white);
	OK->addActionListener(this);

	addAndMakeVisible(Cancel = new NiallsTB(T("Cancel"), T("Boo!")));
	Cancel->setBackgroundColours(tempc, Colours::white);
	Cancel->addActionListener(this);
}

void NPBAddContent::paint(Graphics& g)
{
	SolidColourBrush brush(backColour);
	g.setBrush(&brush);
	g.fillAll();
}

void NPBAddContent::paintOverChildren(Graphics& g)
{
	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(JoyControl->getX(), JoyControl->getY(), JoyControl->getWidth(), JoyControl->getHeight());

	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(MinVal->getX(), MinVal->getY(), MinVal->getWidth(), MinVal->getHeight());

	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(MaxVal->getX(), MaxVal->getY(), MaxVal->getWidth(), MaxVal->getHeight());
}

void NPBAddContent::actionListenerCallback(const String& m)
{
	int i;
	String tempstr;
	NPBMapping *map;

	if(m == OK->getName())
	{
		if(JoyControl->getText() != T("click me"))
		{
			map = new NPBMapping;
			map->JoyIndex = (JoyNum->getSelectedId()-1);
			if(JoyControl->getText().contains(T("Axis")))
			{
				map->JoyControl = evAxis;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Rotation")))
			{
				map->JoyControl = evRotation;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Slider Velocity")))
			{
				map->JoyControl = evSliderVelocity;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider Acceleration")))
			{
				map->JoyControl = evSliderAcceleration;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider Force")))
			{
				map->JoyControl = evSliderForce;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider")))
			{
				map->JoyControl = evSlider;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("POV")))
			{
				map->JoyControl = evPOV;
				if(JoyControl->getText().endsWith(T(" 1")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" 2")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" 3")))
					map->Index = 2;
				else if(JoyControl->getText().endsWith(T(" 4")))
					map->Index = 3;
			}
			else if(JoyControl->getText().contains(T("Button")))
			{
				map->JoyControl = evButton;
				tempstr = JoyControl->getText().fromLastOccurrenceOf(T(" "), false, false);
				map->Index = tempstr.getIntValue();
			}
			else if(JoyControl->getText().contains(T("Angular Velocity")))
			{
				map->JoyControl = evAngularVelocity;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("MIDI Velocity")))
			{
				map->JoyControl = evMIDIVelocity;
				map->Index = 0;
			}
			else if(JoyControl->getText().contains(T("Velocity")))
			{
				map->JoyControl = evVelocity;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Angular Acceleration")))
			{
				map->JoyControl = evAngularAcceleration;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Acceleration")))
			{
				map->JoyControl = evAcceleration;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Force")))
			{
				map->JoyControl = evForce;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Torque")))
			{
				map->JoyControl = evTorque;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Envelope Detector")))
			{
				map->JoyControl = evEnvelopeDetector;
				map->Index = 0;
			}
			else if(JoyControl->getText().contains(T("MIDI CC")))
			{
				map->JoyControl = evMIDICC;
				tempstr = JoyControl->getText().fromLastOccurrenceOf(T(" "), false, false);
				map->Index = tempstr.getIntValue();
			}
			else if(JoyControl->getText().contains(T("MIDI Note Value")))
			{
				map->JoyControl = evMIDINoteVal;
				if(JoyControl->getText().endsWith(T(" On")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" Off")))
					map->Index = 0;
			}
		}
		map->MinVal = MinVal->getText().getFloatValue();
		map->MaxVal = MaxVal->getText().getFloatValue();
		/*if(PluginParameter->getSelectedId() > mixId)
		{
			map->Key = (PluginParameter->getSelectedId()-1) - mixId;
			map->PluginParameter = -1;
		}
		else*/
			map->PluginParameter = (PluginParameter->getSelectedId()-1);
		map->parameterType = true;
		Parent->addMapping(map);
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == Cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	else if(m == JoyControl->getName())
	{
		StringArray joysticks;
		PopupMenu menu1;
		PopupMenu menu2;

		PopupMenu axis;
		axis.addItem(1, T("x"));
		axis.addItem(2, T("y"));
		axis.addItem(3, T("z"));
		menu2.addSubMenu(T("Axis"), axis);

		PopupMenu rot;
		rot.addItem(4, T("x"));
		rot.addItem(5, T("y"));
		rot.addItem(6, T("z"));
		menu2.addSubMenu(T("Rotation"), rot);

		PopupMenu slide;
		slide.addItem(7, T("u"));
		slide.addItem(8, T("v"));
		menu2.addSubMenu(T("Slider"), slide);

		PopupMenu pov;
		pov.addItem(9, T("1"));
		pov.addItem(10, T("2"));
		pov.addItem(11, T("3"));
		pov.addItem(12, T("4"));
		menu2.addSubMenu(T("POV"), pov);

		PopupMenu buttons;
		for(i=0;i<128;i++)
		{
			tempstr.printf("%d", i);
			buttons.addItem((13+i), tempstr);
		}
		menu2.addSubMenu(T("Buttons"), buttons);

		PopupMenu vel;
		vel.addItem(141, T("x"));
		vel.addItem(142, T("y"));
		vel.addItem(143, T("z"));
		menu2.addSubMenu(T("Velocity"), vel);

		PopupMenu angvel;
		angvel.addItem(144, T("x"));
		angvel.addItem(145, T("y"));
		angvel.addItem(146, T("z"));
		menu2.addSubMenu(T("Angular Velocity"), angvel);

		PopupMenu slidevel;
		slidevel.addItem(147, T("u"));
		slidevel.addItem(148, T("v"));
		menu2.addSubMenu(T("Slider Velocity"), slidevel);

		PopupMenu accel;
		accel.addItem(149, T("x"));
		accel.addItem(150, T("y"));
		accel.addItem(151, T("z"));
		menu2.addSubMenu(T("Acceleration"), accel);

		PopupMenu angaccel;
		angaccel.addItem(152, T("x"));
		angaccel.addItem(153, T("y"));
		angaccel.addItem(154, T("z"));
		menu2.addSubMenu(T("Angular Acceleration"), angaccel);

		PopupMenu slideacc;
		slideacc.addItem(155, T("u"));
		slideacc.addItem(156, T("v"));
		menu2.addSubMenu(T("Slider Acceleration"), slideacc);

		PopupMenu force;
		force.addItem(157, T("x"));
		force.addItem(158, T("y"));
		force.addItem(159, T("z"));
		menu2.addSubMenu(T("Force"), force);

		PopupMenu torque;
		torque.addItem(160, T("x"));
		torque.addItem(161, T("y"));
		torque.addItem(162, T("z"));
		menu2.addSubMenu(T("Torque"), torque);

		PopupMenu slidefor;
		slidefor.addItem(163, T("u"));
		slidefor.addItem(164, T("v"));
		menu2.addSubMenu(T("Slider Force"), slidefor);

		menu2.addItem(165, T("Envelope Detector"));

		PopupMenu midicc;
		for(i=0;i<128;i++)
		{
			tempstr.printf("%d", i);
			midicc.addItem((166+i), tempstr);
		}
		menu2.addSubMenu(T("MIDI CC"), midicc);

		PopupMenu midinote;
		midinote.addItem(294, T("On"));
		midinote.addItem(295, T("Off"));
		menu2.addSubMenu(T("MIDI Note Value"), midinote);

		menu2.addItem(296, T("MIDI Velocity"));

		const int result = menu2.show();
		if((result > 12) && (result < 141))
		{
			tempstr.printf("Button %d", (result-13));
			JoyControl->setText(tempstr);
		}
		else if((result > 165) && (result < 294))
		{
			tempstr.printf("MIDI CC %d", (result-166));
			JoyControl->setText(tempstr);
		}
		else
		{
			switch(result) //wish I could think of a better way of doing this...
			{
				case 1:
					JoyControl->setText(T("Axis x"));
					break;
				case 2:
					JoyControl->setText(T("Axis y"));
					break;
				case 3:
					JoyControl->setText(T("Axis z"));
					break;
				case 4:
					JoyControl->setText(T("Rotation x"));
					break;
				case 5:
					JoyControl->setText(T("Rotation y"));
					break;
				case 6:
					JoyControl->setText(T("Rotation z"));
					break;
				case 7:
					JoyControl->setText(T("Slider u"));
					break;
				case 8:
					JoyControl->setText(T("Slider v"));
					break;
				case 9:
					JoyControl->setText(T("POV 1"));
					break;
				case 10:
					JoyControl->setText(T("POV 2"));
					break;
				case 11:
					JoyControl->setText(T("POV 3"));
					break;
				case 12:
					JoyControl->setText(T("POV 4"));
					break;
				case 141:
					JoyControl->setText(T("Velocity x"));
					break;
				case 142:
					JoyControl->setText(T("Velocity y"));
					break;
				case 143:
					JoyControl->setText(T("Velocity z"));
					break;
				case 144:
					JoyControl->setText(T("Angular Velocity x"));
					break;
				case 145:
					JoyControl->setText(T("Angular Velocity y"));
					break;
				case 146:
					JoyControl->setText(T("Angular Velocity z"));
					break;
				case 147:
					JoyControl->setText(T("Slider Velocity u"));
					break;
				case 148:
					JoyControl->setText(T("Slider Velocity v"));
					break;
				case 149:
					JoyControl->setText(T("Acceleration x"));
					break;
				case 150:
					JoyControl->setText(T("Acceleration y"));
					break;
				case 151:
					JoyControl->setText(T("Acceleration z"));
					break;
				case 152:
					JoyControl->setText(T("Angular Acceleration x"));
					break;
				case 153:
					JoyControl->setText(T("Angular Acceleration y"));
					break;
				case 154:
					JoyControl->setText(T("Angular Acceleration z"));
					break;
				case 155:
					JoyControl->setText(T("Slider Acceleration u"));
					break;
				case 156:
					JoyControl->setText(T("Slider Acceleration v"));
					break;
				case 157:
					JoyControl->setText(T("Force x"));
					break;
				case 158:
					JoyControl->setText(T("Force y"));
					break;
				case 159:
					JoyControl->setText(T("Force z"));
					break;
				case 160:
					JoyControl->setText(T("Torque x"));
					break;
				case 161:
					JoyControl->setText(T("Torque y"));
					break;
				case 162:
					JoyControl->setText(T("Torque z"));
					break;
				case 163:
					JoyControl->setText(T("Slider Force u"));
					break;
				case 164:
					JoyControl->setText(T("Slider Force v"));
					break;
				case 165:
					JoyControl->setText(T("Envelope Detector"));
					break;
				case 294:
					JoyControl->setText(T("MIDI Note Value On"));
					break;
				case 295:
					JoyControl->setText(T("MIDI Note Value Off"));
					break;
				case 296:
					JoyControl->setText(T("MIDI Velocity"));
					break;
			}
		}
	}
}

void NPBAddContent::resized()
{
	JoyNum->setBounds(10, 5, 35, 25);
	JoyControl->setBounds(50, 5, 160, 25);
	MinVal->setBounds(220, 5, 50, 25);
	MaxVal->setBounds(280, 5, 50, 25);
	PluginParameter->setBounds(340, 5, 100, 25);

	OK->setBounds(10, (getHeight()-31), 100, 25);
	Cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
}



//---------------------------------------------------------
NPBMappingAddMIDI::NPBMappingAddMIDI(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col):
DialogWindow(T("Add a MIDI Mapping"), col)
{
	setVisible(true);

	setContentComponent(new NPBAddMIDIContent(parent, plugin, col));
}

//---------------------------------------------------------
NPBAddMIDIContent::NPBAddMIDIContent(NPBMappingsContent *parent, NPBPlugin *plugin, Colour col):
Component(),
backColour(col)
{
	int i;
	String tempstr;

	Parent = parent;
	Plugin = plugin;

	addAndMakeVisible(JoyNum = new ComboBox(T("JoyNum")));
	for(i=0;i<Plugin->getNumJoysticks();i++)
	{
		tempstr.printf("%d", i);
		JoyNum->addItem(tempstr, (i+1));
	}
	JoyNum->setSelectedId(1, true);

	addAndMakeVisible(JoyControl = new NiallsLabel(T("JoyControl"), T("click me")));
	JoyControl->addActionListener(this);
	JoyControl->setColours(Colours::black, Colours::white, Colours::black, Colours::white, Colours::black, Colours::white);

	addAndMakeVisible(MinVal = new TextEditor(T("0.0")));
	MinVal->setInputRestrictions(7, T(".0123456789"));
	MinVal->setText(T("0"));

	addAndMakeVisible(MaxVal = new TextEditor(T("1.0")));
	MaxVal->setInputRestrictions(7, T(".0123456789"));
	MaxVal->setText(T("127"));

	addAndMakeVisible(PluginParameter = new ComboBox(T("MIDIParameter")));
	for(i=0;i<128;i++)
	{
		//Plugin->getParameterName(i, tempstr);
		tempstr.printf("MIDI CC %d", i);
		PluginParameter->addItem(tempstr, (i+1));
	}
	tempstr.printf("MIDI Velocity");
	PluginParameter->addItem(tempstr, 129);
	tempstr.printf("MIDI Pitch Bend");
	PluginParameter->addItem(tempstr, 130);
	PluginParameter->setSelectedId(1, true);

	addAndMakeVisible(MIDINote = new ComboBox(T("MIDINote")));
	for(i=0;i<128;i++)
	{
		tempstr.printf("Note %d", i);
		MIDINote->addItem(tempstr, (i+1));
	}
	MIDINote->setSelectedId(1, true);

	Colour tempc = backColour.darker(0.21f);
	Colour temp2 = backColour.darker(0.61f);
	addAndMakeVisible(OK = new NiallsTB(T("OK"), T("Yay!")));
	OK->setBackgroundColours(tempc, Colours::white);
	OK->addActionListener(this);

	addAndMakeVisible(Cancel = new NiallsTB(T("Cancel"), T("Boo!")));
	Cancel->setBackgroundColours(tempc, Colours::white);
	Cancel->addActionListener(this);
}

void NPBAddMIDIContent::paint(Graphics& g)
{
	SolidColourBrush brush(backColour);
	g.setBrush(&brush);
	g.fillAll();
}

void NPBAddMIDIContent::paintOverChildren(Graphics& g)
{
	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(JoyControl->getX(), JoyControl->getY(), JoyControl->getWidth(), JoyControl->getHeight());

	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(MinVal->getX(), MinVal->getY(), MinVal->getWidth(), MinVal->getHeight());

	g.setColour(Colours::black.withAlpha (0.6f));
    g.drawRect(MaxVal->getX(), MaxVal->getY(), MaxVal->getWidth(), MaxVal->getHeight());
}

void NPBAddMIDIContent::actionListenerCallback(const String& m)
{
	int i;
	String tempstr;
	NPBMapping *map;

	if(m == OK->getName())
	{
		if(JoyControl->getText() != T("click me"))
		{
			map = new NPBMapping;
			map->JoyIndex = (JoyNum->getSelectedId()-1);
			if(JoyControl->getText().contains(T("Axis")))
			{
				map->JoyControl = evAxis;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Rotation")))
			{
				map->JoyControl = evRotation;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Slider Velocity")))
			{
				map->JoyControl = evSliderVelocity;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider Acceleration")))
			{
				map->JoyControl = evSliderAcceleration;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider Force")))
			{
				map->JoyControl = evSliderForce;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("Slider")))
			{
				map->JoyControl = evSlider;
				if(JoyControl->getText().endsWith(T(" u")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" v")))
					map->Index = 1;
			}
			else if(JoyControl->getText().contains(T("POV")))
			{
				map->JoyControl = evPOV;
				if(JoyControl->getText().endsWith(T(" 1")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" 2")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" 3")))
					map->Index = 2;
				else if(JoyControl->getText().endsWith(T(" 4")))
					map->Index = 3;
			}
			else if(JoyControl->getText().contains(T("Button")))
			{
				map->JoyControl = evButton;
				tempstr = JoyControl->getText().fromLastOccurrenceOf(T(" "), false, false);
				map->Index = tempstr.getIntValue();
			}
			else if(JoyControl->getText().contains(T("Angular Velocity")))
			{
				map->JoyControl = evAngularVelocity;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Velocity")))
			{
				map->JoyControl = evVelocity;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Angular Acceleration")))
			{
				map->JoyControl = evAngularAcceleration;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Acceleration")))
			{
				map->JoyControl = evAcceleration;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Force")))
			{
				map->JoyControl = evForce;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Torque")))
			{
				map->JoyControl = evTorque;
				if(JoyControl->getText().endsWith(T(" x")))
					map->Index = 0;
				else if(JoyControl->getText().endsWith(T(" y")))
					map->Index = 1;
				else if(JoyControl->getText().endsWith(T(" z")))
					map->Index = 2;
			}
			else if(JoyControl->getText().contains(T("Envelope Detector")))
			{
				map->JoyControl = evEnvelopeDetector;
				map->Index = 0;
			}
		}
		map->MinVal = MinVal->getText().getFloatValue();
		map->MaxVal = MaxVal->getText().getFloatValue();

		map->MIDI = (PluginParameter->getSelectedId()-1);
		map->parameterType = false;
		map->MIDINote = (MIDINote->getSelectedId()-1);
		Parent->addMapping(map);
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	}
	else if(m == Cancel->getName())
		((DialogWindow *)getParentComponent())->closeButtonPressed();
	else if(m == JoyControl->getName())
	{
		StringArray joysticks;
		PopupMenu menu1;
		PopupMenu menu2;

		PopupMenu axis;
		axis.addItem(1, T("x"));
		axis.addItem(2, T("y"));
		axis.addItem(3, T("z"));
		menu2.addSubMenu(T("Axis"), axis);

		PopupMenu rot;
		rot.addItem(4, T("x"));
		rot.addItem(5, T("y"));
		rot.addItem(6, T("z"));
		menu2.addSubMenu(T("Rotation"), rot);

		PopupMenu slide;
		slide.addItem(7, T("u"));
		slide.addItem(8, T("v"));
		menu2.addSubMenu(T("Slider"), slide);

		PopupMenu pov;
		pov.addItem(9, T("1"));
		pov.addItem(10, T("2"));
		pov.addItem(11, T("3"));
		pov.addItem(12, T("4"));
		menu2.addSubMenu(T("POV"), pov);

		PopupMenu buttons;
		for(i=0;i<128;i++)
		{
			tempstr.printf("%d", i);
			buttons.addItem((13+i), tempstr);
		}
		menu2.addSubMenu(T("Buttons"), buttons);

		PopupMenu vel;
		vel.addItem(141, T("x"));
		vel.addItem(142, T("y"));
		vel.addItem(143, T("z"));
		menu2.addSubMenu(T("Velocity"), vel);

		PopupMenu angvel;
		angvel.addItem(144, T("x"));
		angvel.addItem(145, T("y"));
		angvel.addItem(146, T("z"));
		menu2.addSubMenu(T("Angular Velocity"), angvel);

		PopupMenu slidevel;
		slidevel.addItem(147, T("u"));
		slidevel.addItem(148, T("v"));
		menu2.addSubMenu(T("Slider Velocity"), slidevel);

		PopupMenu accel;
		accel.addItem(149, T("x"));
		accel.addItem(150, T("y"));
		accel.addItem(151, T("z"));
		menu2.addSubMenu(T("Acceleration"), accel);

		PopupMenu angaccel;
		angaccel.addItem(152, T("x"));
		angaccel.addItem(153, T("y"));
		angaccel.addItem(154, T("z"));
		menu2.addSubMenu(T("Angular Acceleration"), angaccel);

		PopupMenu slideacc;
		slideacc.addItem(155, T("u"));
		slideacc.addItem(156, T("v"));
		menu2.addSubMenu(T("Slider Acceleration"), slideacc);

		PopupMenu force;
		force.addItem(157, T("x"));
		force.addItem(158, T("y"));
		force.addItem(159, T("z"));
		menu2.addSubMenu(T("Force"), force);

		PopupMenu torque;
		torque.addItem(160, T("x"));
		torque.addItem(161, T("y"));
		torque.addItem(162, T("z"));
		menu2.addSubMenu(T("Torque"), torque);

		PopupMenu slidefor;
		slidefor.addItem(163, T("u"));
		slidefor.addItem(164, T("v"));
		menu2.addSubMenu(T("Slider Force"), slidefor);

		menu2.addItem(165, T("Envelope Detector"));

		const int result = menu2.show();
		switch(result) //wish I could think of a better way of doing this...
		{
			case 1:
				JoyControl->setText(T("Axis x"));
				break;
			case 2:
				JoyControl->setText(T("Axis y"));
				break;
			case 3:
				JoyControl->setText(T("Axis z"));
				break;
			case 4:
				JoyControl->setText(T("Rotation x"));
				break;
			case 5:
				JoyControl->setText(T("Rotation y"));
				break;
			case 6:
				JoyControl->setText(T("Rotation z"));
				break;
			case 7:
				JoyControl->setText(T("Slider u"));
				break;
			case 8:
				JoyControl->setText(T("Slider v"));
				break;
			case 9:
				JoyControl->setText(T("POV 1"));
				break;
			case 10:
				JoyControl->setText(T("POV 2"));
				break;
			case 11:
				JoyControl->setText(T("POV 3"));
				break;
			case 12:
				JoyControl->setText(T("POV 4"));
				break;
			case 141:
				JoyControl->setText(T("Velocity x"));
				break;
			case 142:
				JoyControl->setText(T("Velocity y"));
				break;
			case 143:
				JoyControl->setText(T("Velocity z"));
				break;
			case 144:
				JoyControl->setText(T("Angular Velocity x"));
				break;
			case 145:
				JoyControl->setText(T("Angular Velocity y"));
				break;
			case 146:
				JoyControl->setText(T("Angular Velocity z"));
				break;
			case 147:
				JoyControl->setText(T("Slider Velocity u"));
				break;
			case 148:
				JoyControl->setText(T("Slider Velocity v"));
				break;
			case 149:
				JoyControl->setText(T("Acceleration x"));
				break;
			case 150:
				JoyControl->setText(T("Acceleration y"));
				break;
			case 151:
				JoyControl->setText(T("Acceleration z"));
				break;
			case 152:
				JoyControl->setText(T("Angular Acceleration x"));
				break;
			case 153:
				JoyControl->setText(T("Angular Acceleration y"));
				break;
			case 154:
				JoyControl->setText(T("Angular Acceleration z"));
				break;
			case 155:
				JoyControl->setText(T("Slider Acceleration u"));
				break;
			case 156:
				JoyControl->setText(T("Slider Acceleration v"));
				break;
			case 157:
				JoyControl->setText(T("Force x"));
				break;
			case 158:
				JoyControl->setText(T("Force y"));
				break;
			case 159:
				JoyControl->setText(T("Force z"));
				break;
			case 160:
				JoyControl->setText(T("Torque x"));
				break;
			case 161:
				JoyControl->setText(T("Torque y"));
				break;
			case 162:
				JoyControl->setText(T("Torque z"));
				break;
			case 163:
				JoyControl->setText(T("Slider Force u"));
				break;
			case 164:
				JoyControl->setText(T("Slider Force v"));
				break;
			case 165:
				JoyControl->setText(T("Envelope Detector"));
				break;
			default:
				tempstr.printf("Button %d", (result-13));
				JoyControl->setText(tempstr);
				break;
		}
	}
}

void NPBAddMIDIContent::resized()
{
	JoyNum->setBounds(10, 5, 35, 25);
	JoyControl->setBounds(50, 5, 160, 25);
	MinVal->setBounds(220, 5, 50, 25);
	MaxVal->setBounds(280, 5, 50, 25);
	PluginParameter->setBounds(340, 5, 100, 25);
	MIDINote->setBounds(450, 5, 100, 25);

	OK->setBounds(10, (getHeight()-31), 100, 25);
	Cancel->setBounds((getWidth()-110), (getHeight()-31), 100, 25);
}

/*void NPBAddContent::setupComboBox(ComboBox *cb, int index)
{
	index++;
	PluginParameter->addItem(T("Key a"), index);
	index++;
	PluginParameter->addItem(T("Key b"), index);
	index++;
	PluginParameter->addItem(T("Key c"), index);
	index++;
	PluginParameter->addItem(T("Key d"), index);
	index++;
	PluginParameter->addItem(T("Key e"), index);
	index++;
	PluginParameter->addItem(T("Key f"), index);
	index++;
	PluginParameter->addItem(T("Key g"), index);
	index++;
	PluginParameter->addItem(T("Key h"), index);
	index++;
	PluginParameter->addItem(T("Key i"), index);
	index++;
	PluginParameter->addItem(T("Key j"), index);
	index++;
	PluginParameter->addItem(T("Key k"), index);
	index++;
	PluginParameter->addItem(T("Key l"), index);
	index++;
	PluginParameter->addItem(T("Key m"), index);
	index++;
	PluginParameter->addItem(T("Key n"), index);
	index++;
	PluginParameter->addItem(T("Key o"), index);
	index++;
	PluginParameter->addItem(T("Key p"), index);
	index++;
	PluginParameter->addItem(T("Key q"), index);
	index++;
	PluginParameter->addItem(T("Key u"), index);
	index++;
	PluginParameter->addItem(T("Key r"), index);
	index++;
	PluginParameter->addItem(T("Key s"), index);
	index++;
	PluginParameter->addItem(T("Key t"), index);
	index++;
	PluginParameter->addItem(T("Key u"), index);
	index++;
	PluginParameter->addItem(T("Key v"), index);
	index++;
	PluginParameter->addItem(T("Key w"), index);
	index++;
	PluginParameter->addItem(T("Key x"), index);
	index++;
	PluginParameter->addItem(T("Key y"), index);
	index++;
	PluginParameter->addItem(T("Key z"), index);
}*/
