//	NPBPluginEditor.cpp - Simple editor for plugins which don't have
//						  their own gui.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBPluginEditor.h"

NPBPluginEditor::NPBPluginEditor(AEffect *plugin, NPBPlugin *parent, Colour col):
DialogWindow(T("NPBPluginEditor"), col)
{
	char tempstr[256];
	sprintf(tempstr, "");

	Parent = parent;

	plugin->dispatcher(plugin, effGetEffectName, 0, 0, tempstr, 0.0f);
	setName(tempstr);

	Viewport *view = new Viewport();
	NPBPluginEditorContent *cont = new NPBPluginEditorContent(plugin, parent);
	view->setViewedComponent(cont);
	setContentComponent(view);
	setVisible(true);
	setResizable(true, false);
}

NPBPluginEditor::~NPBPluginEditor()
{
	Parent->EditorOpen = false;	
	Parent->setEditor(0);
}

void NPBPluginEditor::setMix(float val)
{
	Viewport *view;
	NPBPluginEditorContent *content;
	view = static_cast<Viewport *>(getContentComponent());
	content = static_cast<NPBPluginEditorContent *>(view->getViewedComponent());

	content->setMix(val);
}

void NPBPluginEditor::setParameter(long index, float val)
{
	Viewport *view;
	NPBPluginEditorContent *content;
	view = static_cast<Viewport *>(getContentComponent());
	content = static_cast<NPBPluginEditorContent *>(view->getViewedComponent());

	content->setParameter(index, val);
}

//-------------------------------------------------------------------------
NPBPluginEditorContent::NPBPluginEditorContent(AEffect *plugin, NPBPlugin *parent):Component()
{
	int i;
	int tempint;
	char tempstr[256];
	Sliders = 0;
	Plugin = plugin;
	Parent = parent;

	addAndMakeVisible(Programs = new ComboBox(T("Programs")));
	Programs->addActionListener(this);
	Programs->setBounds(108, 0, 103, 20);
	if(Plugin->dispatcher(Plugin, effGetProgramNameIndexed, 0, -1, tempstr, 0.0f))
	{
		for(i=0;i<Plugin->numPrograms;i++)
		{
			Plugin->dispatcher(Plugin, effGetProgramNameIndexed, i, -1, tempstr, 0.0f);
			Programs->addItem(tempstr, i);
			sprintf(tempstr, "");
		}
		Programs->setSelectedId(Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f), false);
	}
	else
	{
		tempint = Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f);
		//the hard way - have to set each program to find it's name
		for(i=0;i<Plugin->numPrograms;i++)
		{
			Plugin->dispatcher(Plugin, effSetProgram, 0, i, tempstr, 0.0f);
			Plugin->dispatcher(Plugin, effGetProgramName, 0, 0, tempstr, 0.0f);
			Programs->addItem(tempstr, i);
			sprintf(tempstr, "");
		}
		Plugin->dispatcher(Plugin, effSetProgram, 0, tempint, tempstr, 0.0f);
		Programs->setSelectedId(Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f), false);
	}

	addAndMakeVisible(MixLabel = new Label(T("MixLabel"), T("Mix :")));
	MixLabel->setBounds(216, 0, 37, 20);

	addAndMakeVisible(Mix = new Slider(T("Mix")));
	Mix->addChangeListener(this);
	//Mix->setTextBoxProportionalSize(0.0f);
	Mix->setTextBoxStyle(Slider::NoTextBox, true, 0, 0);
	Mix->setRange(0.0, 1.0);
	Mix->setBounds(258, 0, 130, 20);
	Mix->setValue((float)Parent->getMix());

	Labels = new Label *[Plugin->numParams];
	Sliders = new Slider *[Plugin->numParams];
	Units = new Label *[Plugin->numParams];
	for(i=0;i<Plugin->numParams;i++)
	{
		Plugin->dispatcher(Plugin, effGetParamName, i, 0, tempstr, 0.0f);
		addAndMakeVisible(Labels[i] = new Label(tempstr, tempstr));
		Labels[i]->setBounds(5, (25+(i*25)), 150, 20);
		sprintf(tempstr, "");

		sprintf(tempstr, "Slider %d", i);
		addAndMakeVisible(Sliders[i] = new Slider(tempstr));
		Sliders[i]->addChangeListener(this);
		//Sliders[i]->setTextBoxProportionalSize(0.0f);
		Sliders[i]->setTextBoxStyle(Slider::NoTextBox, true, 0, 0);
		Sliders[i]->setRange(0.0, 1.0);
		Sliders[i]->setBounds(160, (25+(i*25)), 200, 20);
		Sliders[i]->setValue((double)Plugin->getParameter(Plugin, i));
		sprintf(tempstr, "");

		Plugin->dispatcher(Plugin, effGetParamLabel, i, 0, tempstr, 0.0f);
		addAndMakeVisible(Units[i] = new Label(tempstr, tempstr));
		Units[i]->setBounds(365, (25+(i*25)), 60, 20);
		sprintf(tempstr, "");
	}
	setSize(435, (30+(i*25)));
}

NPBPluginEditorContent::~NPBPluginEditorContent()
{
	delete [] Labels;
	delete [] Sliders;
	delete [] Units;
}

void NPBPluginEditorContent::actionListenerCallback(const String& m)
{
	int i;

	if(m == Programs->getName())
	{
		Plugin->dispatcher(Plugin, effSetProgram, 0, Programs->getSelectedId(), NULL, 0.0f);
		for(i=0;i<Plugin->numParams;i++)
			Sliders[i]->setValue(static_cast<double>(Plugin->getParameter(Plugin, i)));
	}
}

void NPBPluginEditorContent::changeListenerCallback(void *objectThatHasChanged)
{
	int i;

	if(objectThatHasChanged == Mix)
		Parent->setMix(static_cast<float>(Mix->getValue()));
	else
	{
		for(i=0;i<Plugin->numParams;i++)
		{
			if(objectThatHasChanged == Sliders[i])
			{
				Plugin->setParameter(Plugin, i, static_cast<float>(Sliders[i]->getValue()));
				break;
			}
		}
	}
}

void NPBPluginEditorContent::setParameter(long index, float val)
{
	if(Sliders)
		Sliders[index]->setValue(static_cast<double>(val));
}