//	NiallsTB.cpp - A TextButton fixing the drawing of the
//				   TextButton class changes introduced in JUCE v1.12.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include <juce.h>
#include "NiallsTB.h"

NiallsTB::NiallsTB(const String &buttonName, const String &toolTip):
TextButton(buttonName, toolTip)
{
	
}

void NiallsTB::paintButton (Graphics& g,
                              bool isMouseOverButton,
                              bool isButtonDown)
{
    getLookAndFeel().drawButtonBackground (g, *this, getBackgroundColour(),
                                           isMouseOverButton,
                                           isButtonDown);

    const int yIndent = jmin (4, proportionOfHeight (0.3f));
    const int cornerSize = jmin (getHeight(), getWidth()) / 2;

    g.setColour (getBackgroundColour().darker(0.7f));
    g.setFont (getFont());

    if (! isEnabled())
        g.setOpacity (0.5f);

    const int leftIndent = 2 + cornerSize / (isConnectedOnLeft() ? 4 : 2);
    const int rightIndent = 2 + cornerSize / (isConnectedOnRight() ? 4 : 2);

    g.drawFittedText (getButtonText(),
                      leftIndent,
                      yIndent,
                      getWidth() - leftIndent - rightIndent,
                      getHeight() - yIndent * 2,
                      Justification::centred, 2);
}