//	ParameterFilter.h - A filter to smooth parameter changes, taken from music
//						dsp.org:
//						http://www.musicdsp.org/archive.php?classid=3#28
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef PARAMETERFILTER_H_
#define PARAMETERFILTER_H_

///	A simple filter used to filter joystick events.
/*!
	Taken from musicdsp.org:
	http://www.musicdsp.org/archive.php?classid=3#28
 */
class ParameterFilter
{
  public:
	///	Constructor.
	ParameterFilter(float samplerate, double scale=0.0001); //0.0022675736961451247165532879818594=100/44100
	///	Destructor.
	~ParameterFilter() {};

	///	Returns the current (or next) output of the filter.
	float GetSample(float inval);

	void SetScale(double samples);	//takes a value 0->1
	void SetSmooth(double val);		//ditto
	void SetSamplerate(float samplerate);
  private:
	//--parameters--
	double Scale;
	double Smoothness;
	float Samplerate;

	//--precalc variables--
	double A,
		   B;
	double ACoeff,
		   BCoeff,
		   CCoeff;
	double MasterGain;
	double AGain,
		   BGain,
		   CGain;

	//--runtime variables--
	double AReg, BReg, CReg;
};

#endif
