//	VerticalSlider.cpp - Like Jules' Slider, only vertical (I couldn't see a
//					     way of doing this by subclassing...).
//						 **This is a hack.  You probably don't want to use it
//						   in your own code.**
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

/*#include "../../../../juce_core/basics/juce_StandardHeader.h"


namespace JUCE {*/

#include "VerticalSlider.h"


//==============================================================================
static double skew (const double n, const double skewFactor)
{
    return pow (n, skewFactor);
}

static double unskew (const double n, const double skewFactor)
{
    return exp (log (n) / skewFactor);
}


//==============================================================================
VerticalSlider::VerticalSlider (const String& name,
                const bool upDownButtons)
  : Component (name),
    currentValue (0.0),
    minimum (0),
    maximum (10),
    interval (0),
    skewFactor (1.0),
    endlessStart (0.0),
    endlessScale (0.0),
    numDecimalPlaces (3),
    sliderRegionLeft (0),
    textBoxProportion (0.3f),
    enabled (true),
    editableText (true),
    usesUpDownButtons (upDownButtons),
    isVelocityBased (false),
    isEndless (false),
    upButton (0),
    downButton (0)
{
    addAndMakeVisible (valueBox = new Label (T("number"),
                                             T("number")));

    valueBox->setEditable (editableText && enabled);
    valueBox->addTextEditorListener (this);

    setWantsKeyboardFocus (false);
    setRepaintsOnMouseActivity (true);

    lookAndFeelChanged();
    updateText();
}

VerticalSlider::~VerticalSlider()
{
    deleteAllChildren();
}

void VerticalSlider::lookAndFeelChanged()
{
    deleteAllChildren();
    addAndMakeVisible (valueBox = new Label (T("number"),
                                             T("number")));

    valueBox->setEditable (editableText && enabled);
    valueBox->addTextEditorListener (this);

    if (usesUpDownButtons)
    {
        addAndMakeVisible (upButton = getLookAndFeel().createSliderButton(true));
        upButton->addActionListener (this);
        upButton->setRepeatSpeed (300, 100);

        addAndMakeVisible (downButton = getLookAndFeel().createSliderButton(false));
        downButton->addActionListener (this);
        downButton->setRepeatSpeed (300, 100);

        textBoxProportion = 0.6f;
    }

    Colour background, sliderColour, textBackground, textForeground, textHighlight;

    getLookAndFeel().getSliderColours (background,
                                       sliderColour,
                                       textBackground,
                                       textForeground,
                                       textHighlight);

    valueBox->setColours (textForeground,
                          textBackground,
                          textForeground,
                          textBackground,
                          textHighlight,
                          Colour::greyLevel (0.5f));

    setComponentEffect (getLookAndFeel().getSliderEffect());
}

void VerticalSlider::setRange (const double newMin,
                       const double newMax,
                       const double newInt)
{
    if (minimum != newMin
        || maximum != newMax
        || interval != newInt)
    {
        minimum = newMin;
        maximum = newMax;
        interval = newInt;

        // figure out the number of DPs needed to display all values at this
        // interval setting.
        numDecimalPlaces = 3;

        if (newInt != 0)
        {
            int v = abs ((int)(newInt * 10000000));

            while ((v % 10) == 0)
            {
                --numDecimalPlaces;
                v /= 10;
            }
        }

        setValue (currentValue, false, false);
        updateText();
    }
}

void VerticalSlider::setValue (double newValue,
                       const bool sendUpdateMessage,
                       const bool sendMessageSynchronously)
{
    if (newValue <= minimum || maximum <= minimum)
    {
        newValue = minimum;
    }
    else if (newValue >= maximum)
    {
        newValue = maximum;
    }
    else if (interval > 0)
    {
        newValue = minimum + interval * floor ((newValue - minimum) / interval + 0.5);
    }

    if (currentValue != newValue)
    {
        currentValue = newValue;
        updateText();
        repaint();

        if (sendUpdateMessage)
        {
            if (sendMessageSynchronously)
                sendSynchronousChangeMessage (this);
            else
                sendChangeMessage (this);
        }
    }
}

void VerticalSlider::updateText()
{
    valueBox->setText (getTextFromValue (currentValue));
}

const String VerticalSlider::getTextFromValue (double v)
{
    if (numDecimalPlaces > 0)
        return String (v, numDecimalPlaces);
    else
        return String ((int) v);
}

double VerticalSlider::getValueFromText (const String& text)
{
    return text.trimStart()
               .initialSectionContainingOnly (T("0123456789.-"))
               .getDoubleValue();
}

void VerticalSlider::setLabelIsEditable (const bool shouldBeEditable) throw()
{
    editableText = shouldBeEditable;
    valueBox->setEditable (shouldBeEditable && enabled);
}

void VerticalSlider::setVelocityBasedMode (const bool velBased) throw()
{
    isVelocityBased = velBased;
}

void VerticalSlider::setSkewFactor (const double factor) throw()
{
    skewFactor = factor;
}

void VerticalSlider::setEndless (bool shouldBeEndless,
                         double startValue,
                         double valueChangePerPixelMoved)
{
    isEndless = shouldBeEndless;
    endlessStart = startValue;
    endlessScale = valueChangePerPixelMoved;
    setValue (endlessStart);
}

void VerticalSlider::setTextBoxProportionalSize (const float p)
{
    textBoxProportion = jlimit (0.0f, 1.0f, p);
    resized();
}

void VerticalSlider::setEnabled (const bool shouldBeEnabled)
{
    if (enabled != shouldBeEnabled)
    {
        enabled = shouldBeEnabled;
        valueBox->setEditable (shouldBeEnabled);

        if (upButton != 0)
            upButton->setEnabled (shouldBeEnabled);

        if (downButton != 0)
            downButton->setEnabled (shouldBeEnabled);

        repaint();
    }
}

bool VerticalSlider::isEnabled() const throw()
{
    return enabled;
}

void VerticalSlider::startedDragging()
{
}

void VerticalSlider::stoppedDragging()
{
}

//==============================================================================
void VerticalSlider::textChange (const bool retPressed,
                         const bool escPressed)
{
    if(retPressed || !valueBox->hasKeyboardFocus(true))
    {
        startedDragging();
        setValue(getValueFromText(valueBox->getText()), true, true);
        stoppedDragging();
    }

    if((retPressed||escPressed) && valueBox->hasKeyboardFocus(true))
    {
        grabKeyboardFocus();
        updateText();
    }
}

void VerticalSlider::textEditorTextChanged (TextEditor& editor)
{
    textChange (false, false);
}

void VerticalSlider::textEditorReturnKeyPressed (TextEditor& editor)
{
    textChange (true, false);
}

void VerticalSlider::textEditorEscapeKeyPressed (TextEditor& editor)
{
    textChange (false, true);
}

void VerticalSlider::textEditorFocusLost (TextEditor& editor)
{
    textChange (false, true);
}

void VerticalSlider::actionListenerCallback (const String& m)
{
    if (usesUpDownButtons)
    {
        startedDragging();

        if (m == upButton->getName())
            setValue (getValue() + interval, true, true);
        else if (m == downButton->getName())
            setValue (getValue() - interval, true, true);

        stoppedDragging();
    }
}

//==============================================================================
void VerticalSlider::paint (Graphics& g)
{
    if (! usesUpDownButtons)
    {
        if (isEndless)
        {
            drawEndlessSlider (g,
                               0, sliderRegionLeft,
                               getWidth(), sliderRegionWidth);
        }
        else
        {
            float sliderX = 0.0f;

            if (maximum > minimum)
            {
                if (currentValue <= minimum)
                {
                    sliderX = (float)(/*sliderRegionLeft + */sliderRegionWidth);
					//sliderX = 8.0f;
                }
                else if (currentValue >= maximum)
                {
                    //sliderX = (float)(sliderRegionLeft + sliderRegionWidth);
					//sliderX = (float)sliderRegionWidth;
					sliderX = 8.0f;
                }
                else
                {
                    /*sliderX = (float)(sliderRegionLeft + sliderRegionWidth
                                       * skew ((currentValue - minimum) / (maximum - minimum),
                                               skewFactor));
					sliderX = (float)(sliderRegionWidth
                                       * skew ((currentValue - minimum) / (maximum - minimum),
                                               skewFactor));*/
					double tempf = (maximum-minimum);
					sliderX = (float)(sliderRegionWidth*skew(tempf-((currentValue-minimum)/tempf), skewFactor));
					sliderX -= sliderRegionWidth;
                }
            }
            else
            {
                //sliderX = sliderRegionLeft + 0.5f * sliderRegionWidth;
				sliderX = 0.5f * sliderRegionWidth;
            }

            /*drawSlider (g,
                        0, sliderRegionLeft,
                        getWidth(), sliderRegionWidth,
                        sliderX);*/
			drawSlider (g,
                        0, 8,
                        getWidth(), sliderRegionWidth-8,
                        sliderX);
        }
    }
}

void VerticalSlider::resized()
{
    const int minW = jmin (64, proportionOfHeight (0.6f));

    if (textBoxProportion > 0.0f)
    {
        valueBox->setVisible (true);
        sliderRegionLeft = proportionOfWidth (jmax (textBoxProportion, minW / (float)getHeight()));
    }
    else
    {
        valueBox->setVisible (false);
        sliderRegionLeft = 8;
    }

    sliderRegionWidth = (getHeight() - 8) - sliderRegionLeft;
    valueBox->setBounds (0, (getHeight()-sliderRegionLeft), getWidth(), sliderRegionLeft-10);

    //Font f;
    //getLookAndFeel().getSliderFont (f, *this);
    //valueBox->setFont (f);

    if (usesUpDownButtons)
    {
        const int buttonW = sliderRegionWidth / 2 - 2;

        downButton->setBounds (sliderRegionLeft,
                               proportionOfHeight (0.2f),
                               buttonW,
                               proportionOfHeight (0.6f));

        upButton->setBounds (sliderRegionLeft + buttonW + 2,
                             proportionOfHeight (0.2f),
                             buttonW,
                             proportionOfHeight (0.6f));
    }
}

void VerticalSlider::mouseDown (const MouseEvent& e)
{
    if (enabled && ((maximum > minimum) && ! usesUpDownButtons))
    {
        valueBox->hideEditor (true);

        mouseXWhenLastDragged = e.x;
        valueWhenLastDragged = currentValue;
        startedDragging();
    }
}

void VerticalSlider::mouseUp (const MouseEvent&)
{
    if (enabled)
    {
        if ((maximum > minimum) && ! usesUpDownButtons)
            stoppedDragging();

        if (isValidComponent() && isEndless)
            setValue (endlessStart);
    }
}

void VerticalSlider::mouseDrag (const MouseEvent& e)
{
    if(enabled)
    {
        if(isEndless)
        {
            const double delta = jlimit(minimum, maximum,
                                         e.getDistanceFromDragStartX() * endlessScale);

            if(delta != currentValue)
                setValue(delta, true, false);
        }
        else if ((maximum > minimum) && !usesUpDownButtons)
        {
            if ((!(isVelocityBased || e.mods.isAnyModifierKeyDown()))
                 || ((maximum - minimum)/sliderRegionWidth < interval))
            {
                const float y = (float)e.y;

                if(y <= 8)
                {
                    valueWhenLastDragged = minimum;
					//valueWhenLastDragged = maximum;
                }
                else if(y >= (sliderRegionLeft + sliderRegionWidth))
                {
                    valueWhenLastDragged = maximum;
					//valueWhenLastDragged = minimum;
                }
                else
                {
                    valueWhenLastDragged = minimum + ((maximum - minimum)*unskew(y/sliderRegionWidth, skewFactor));
                }

                setValue(((maximum-minimum)-(valueWhenLastDragged-minimum)), true, false);
				//setValue(valueWhenLastDragged, true, false);
                mouseXWhenLastDragged = e.y;
            }
            else
            {
                const double maxSpeed = sliderRegionWidth;

                double speed = jlimit(-maxSpeed, maxSpeed,
                                       (double)(e.x - mouseXWhenLastDragged));

                if(speed != 0)
                {
                    speed = jmax (0.0, speed * speed - 2.0) / 2;
                    double delta = 0.5 * interval + speed * (maximum - minimum)/(maxSpeed * maxSpeed);
                    if(e.x < mouseXWhenLastDragged)
                        delta = -delta;

                    const double v = jlimit(minimum, maximum, valueWhenLastDragged + delta);

                    setValue(v, true, false);
                    mouseXWhenLastDragged = e.x;
                    valueWhenLastDragged = v;
                }
            }
        }
    }
}

void VerticalSlider::mouseDoubleClick (const MouseEvent&)
{
    if (enabled
         && (! usesUpDownButtons)
         && (minimum <= 0 && maximum >= 0))
    {
        startedDragging();
        setValue (0, true, true);
        stoppedDragging();
    }
}

void VerticalSlider::mouseWheelMove (const MouseEvent& e, float increment)
{
    if (enabled
         && (maximum > minimum)
         && ! isMouseButtonDownAnywhere())
    {
        valueBox->hideEditor (false);

        const double delta = jmax ((maximum - minimum) * 0.04f * fabsf (increment),
                                   interval);

        startedDragging();

        setValue ((increment >= 0) ? currentValue + delta
                                   : currentValue - delta,
                  true, true);

        stoppedDragging();
    }
}

void VerticalSlider::drawEndlessSlider (Graphics& g,
                                        int x, int y,
                                        int w, int h)
{
    Colour background, sliderColour, textBackground, textForeground, textHighlight;
    getLookAndFeel().getSliderColours (background, sliderColour, textBackground, textForeground, textHighlight);

    g.setColour (background);
    g.fillRect (x, y, w, h);

    if (isEnabled())
    {
        if (isMouseButtonDown())
        {
            g.setColour (Colours::red.withAlpha (0.7f));
        }
        else
        {
            if (isMouseOver())
                g.setColour (sliderColour);
            else
                g.setColour (sliderColour.withAlpha (0.7f));
        }
    }
    else
    {
        g.setColour (sliderColour.withAlpha (0.5f));
    }

    g.drawArrow (x + w * 0.3f, y + h * 0.5f, x + w * 0.1f, y + h * 0.5f, h * 0.1f, h * 0.3f, h * 0.4f);
    g.drawArrow (w + w * 0.7f, y + h * 0.5f, x + w * 0.9f, y + h * 0.5f, h * 0.1f, h * 0.3f, h * 0.4f);

    g.setFont (h * 0.5f, Font::bold | Font::italic);
    g.drawText (T("drag"), x, y, w, h, Justification::centred, false);
}

void VerticalSlider::drawSlider (Graphics& g,
                                 int x, int y,
                                 int w, int h,
                                 float sliderX)
{
    Colour background, sliderColour, textBackground, textForeground, textHighlight;
    getLookAndFeel().getSliderColours (background, sliderColour, textBackground, textForeground, textHighlight);

    g.setColour (background);
    g.fillRect (x, y, w, h);

    g.setColour (Colours::grey.withAlpha (isEnabled() ? 0.5f : 0.15f));

    g.fillRect (x + roundFloatToInt (w * 0.6f), y,
                roundFloatToInt (w * 0.125f), h);

    float alpha = 0.35f;

    if (isEnabled())
        alpha = isMouseOverOrDragging() ? 1.0f : 0.7f;

    g.setColour (sliderColour.withAlpha (alpha));

    Path p;
    /*p.addTriangle (sliderX, y + h * 0.9f,
                   sliderX - 7.0f, y + h * 0.2f,
                   sliderX + 7.0f, y + h * 0.2f);*/
	p.addTriangle (x + w * 0.8f, sliderX,
                   x + w * 0.5f, sliderX - 7.0f,
                   x + w * 0.5f, sliderX + 7.0f);

    g.fillPath (p);

    g.setColour (Colours::black.withAlpha (isEnabled() ? 0.7f : 0.35f));
    g.strokePath (p, PathStrokeType (0.3f));
}