//	VerticalSlider.h - Like Jules' Slider, only vertical (I couldn't see a way
//					   of doing this by subclassing...).
//					   **This is a hack.  You probably don't want to use it
//					     in your own code.**
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef VERTICALSLIDER_H_
#define VERTICALSLIDER_H_

#include <juce.h>

///	A vertical version of Slider.
/*!
	This is *very* poorly done on my part - I just copied Jules' Slider code
	and flipped it round (you'll notice most of the variables are still
	called x, when they should be y...).

	(I haven't commented any of the methods, because they're all Jules')
 */
class VerticalSlider  : public Component,
                public ChangeBroadcaster,
                private ActionListener,
                private TextEditorListener
{
public:
    VerticalSlider(const String& componentName,
				   const bool usesUpDownButtons = false);
    ~VerticalSlider();

    void setRange(const double newMinimum,
                  const double newMaximum,
                  const double newInterval = 0);
    void setValue(double newValue,
                  const bool sendUpdateMessage = true,
                  const bool sendMessageSynchronously = false);
    inline double getValue() const     {return currentValue;};
    inline double getMaximum() const   {return maximum;};
    inline double getMinimum() const   {return minimum;};

    void setVelocityBasedMode(const bool isVelocityBased) throw();
    void setEndless(bool shouldBeEndless,
                    double startValue,
                    double valueChangePerPixelMoved);
    void setLabelIsEditable(const bool shouldBeEditable) throw();
    void setTextBoxProportionalSize(const float proportion);
    void setSkewFactor(const double factor) throw();

    void setEnabled(const bool shouldBeEnabled);
    bool isEnabled() const throw();

    virtual void startedDragging();
    virtual void stoppedDragging();

    virtual double getValueFromText(const String& text);
    virtual const String getTextFromValue(double value);

    void updateText();

    juce_UseDebuggingNewOperator
protected:
    void textEditorTextChanged(TextEditor& editor);
    void textEditorReturnKeyPressed(TextEditor& editor);
    void textEditorEscapeKeyPressed(TextEditor& editor);
    void textEditorFocusLost(TextEditor& editor);
    void paint(Graphics& g);
    void resized();
    void mouseDown(const MouseEvent& e);
    void mouseUp(const MouseEvent& e);
    void mouseDrag(const MouseEvent& e);
    void mouseDoubleClick(const MouseEvent& e);
    void mouseWheelMove(const MouseEvent& e, float wheelIncrement);
    void actionListenerCallback(const String& message);
    void lookAndFeelChanged();
	void drawEndlessSlider(Graphics& g, int x, int y, int w, int h);
	void drawSlider(Graphics& g, int x, int y, int w, int h, float sliderX);

private:
    double currentValue, minimum, maximum, interval;
    double valueWhenLastDragged, skewFactor, endlessStart, endlessScale;
    int numDecimalPlaces, mouseXWhenLastDragged;
    int sliderRegionLeft, sliderRegionWidth;
    float textBoxProportion;
    bool enabled, editableText, usesUpDownButtons, isVelocityBased, isEndless;
    Font font;
    Label* valueBox;
    Button* upButton;
    Button* downButton;

    void textChange(const bool retPressed, const bool escPressed);
    Slider(const Slider&);
};


#endif
