///	FileMenu.cpp - The program's File menu.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------
///	
///	You can e-mail me at niallmoody@yahoo.co.uk if you want.
///	
///	--------------------------------------------------------------------------

#include "../NPBWindow.h"
#include "FileMenu.h"
#include "../ComponentFactory.h"
#include "../NiallsLabel.h"

//--static member variables must be declared--
String FileMenuComp::text = T("File");
int FileMenuComp::red = 0;
int FileMenuComp::green = 0;
int FileMenuComp::blue = 0;

///---------------------------------------------------------------------------
FileMenuComp::FileMenuComp(ComponentFactory *factory, Component **windowComponent):
FactoryObject(factory, windowComponent)
{
	//--Set default values to allow for the user missing out on an attribute.--
	x = 10;
	y = 32;
	xRel = -1;
	yRel = -1;
	width = 29;
	height = 25;
	widthRel = -1;
	heightRel = -1;

	//Register this object with the factory.
	factory->registerObject(this, String("fileMenu"));

	//Register all it's attributes with the factory.
	factory->registerStringAttribute(this, String("text"), &FileMenuComp::setText);
	factory->registerIntAttribute(this, String("colour_red"), &FileMenuComp::setColourRed);
	factory->registerIntAttribute(this, String("colour_green"), &FileMenuComp::setColourGreen);
	factory->registerIntAttribute(this, String("colour_blue"), &FileMenuComp::setColourBlue);
}

///---------------------------------------------------------------------------
void FileMenuComp::create()
{
	NPBWindow *window;
	NiallsLabel *tempComp;
	Colour tempcol(red, green, blue);

	window = static_cast<NPBWindow *>(factory->getWindow());

	window->addAndMakeVisible(tempComp = new NiallsLabel(T("FileMenu"), text));
	tempComp->setColours(tempcol,
						 Colours::transparentBlack,
						 Colours::transparentBlack,
						 Colours::transparentBlack,
						 Colours::transparentBlack,
						 Colours::transparentBlack);
	tempComp->addActionListener(window);

	*windowComponent = tempComp;
}

///---------------------------------------------------------------------------
void FileMenuComp::setText(String val)
{
	text = val;
}

///---------------------------------------------------------------------------
void FileMenuComp::setColourRed(int val)
{
	red = val;
}

///---------------------------------------------------------------------------
void FileMenuComp::setColourGreen(int val)
{
	green = val;
}

///---------------------------------------------------------------------------
void FileMenuComp::setColourBlue(int val)
{
	blue = val;
}
